package com.hrsk.cloud.eg.domain.api;

import com.alibaba.cola.domain.Entity;
import com.alibaba.cola.exception.BizException;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.google.gson.JsonObject;
import com.hrsk.cloud.eg.domain.api.common.ApiPayloadTypeEnum;
import com.hrsk.cloud.eg.domain.api.resolver.payload.ApiPayloadResolverManage;
import com.hrsk.cloud.eg.domain.api.resolver.result.ApiResultResolverManage;
import com.hrsk.cloud.eg.vo.response.DataVo;
import com.hrsk.cloud.eg.vo.response.DefaultResponseVo;
import lombok.Getter;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author: bianlanzhou
 * @create: 2024-07-26 17:59
 * @description: HTTP API
 **/
@Slf4j
@ToString
@Entity
public class HttpApi implements Api {
    /**
     * url分隔符
     */
    private static final String URL_SEPARATOR = "/";

    /**
     * search分隔符
     */
    private static final String URL_SEARCH_SEPARATOR = "&";

    /**
     * 等于号
     */
    private static final String _EQUAL = "=";
    /**
     * search标记符
     */
    private static final String URL_SEARCH = "?";

    @Override
    public String getType() {
        return "http";
    }

    /**
     * url地址
     */
    @Getter
    private String url;
    /**
     * 请求方法
     */
    @Getter
    private String method;
    /**
     * http api配置
     */
    private HttpApiConfig httpApiConfig;
    /**
     * header数据
     */
    @Getter
    private final Map<String, Object> header = new HashMap<>();
    /**
     * 请求payload数据
     */
    @Getter
    private Object payload;


    /**
     * 加载
     * @param httpApiConfig http api 配置
     */
    public void load(HttpApiConfig httpApiConfig,Object input){
        this.httpApiConfig = httpApiConfig;
        ApiContext context = new ApiContext(input);
        this.method = httpApiConfig.getMethod();
        loadCompleteUrl(context);
        loadHeader(context);
        loadBody(context);
    }

    /**
     * @description: 加载返回值配置
     * @author zhangyy
     * @date 2024/9/2 15:58
     * @version 1.0
     */
    public Object loadResult(HttpApiConfig config,Object output){
        this.httpApiConfig = config;
         Map<String,Object> params =JSONObject.parseObject(output.toString(),Map.class);
         return doLoadResult(params);
    }


    /**
     * 加载path,包含path和search
     * @param context 上下文
     */
    private void loadCompleteUrl(ApiContext context){
        StringBuilder urlSb = new StringBuilder(httpApiConfig.getUrl());
        for(ValueObject valueObject : httpApiConfig.getPath()){
            urlSb.append(URL_SEPARATOR).append(valueObject.compile(context).getRealValue());
        }
        if(!CollectionUtils.isEmpty(httpApiConfig.getSearch())){
            StringBuilder searchSb = new StringBuilder();
            for(KeyValueObject keyValueObject : httpApiConfig.getSearch()){
                searchSb.append(URL_SEARCH_SEPARATOR).append(keyValueObject.getKey()).append(_EQUAL).append(keyValueObject.compile(context).getRealValue());
            }
            urlSb.append(URL_SEARCH).append(searchSb.toString().replaceFirst(URL_SEARCH_SEPARATOR,""));
        }
        url = urlSb.toString();
    }

    /**
     * 加载header
     * @param context 上下文
     */
    private void loadHeader(ApiContext context){
        for(KeyValueObject keyValueObject : httpApiConfig.getHeader()){
            this.header.put(keyValueObject.getKey(),  keyValueObject.compile(context).getRealValue().toString());
        }
    }




    /**
     * 加载请求体
     * @param context 上下文
     */
    private void loadBody(ApiContext context){
        if(StringUtils.equals(httpApiConfig.getContentType(), ApiPayloadTypeEnum.APPLICATION_JSON.getCode())
        ||StringUtils.equals(httpApiConfig.getContentType(),ApiPayloadTypeEnum.APPLICATION_X_WWW_FORM_URLENCODED.getCode())
        ){
            payload = Objects.toString(ApiPayloadResolverManage.getInstance().resolve(httpApiConfig.getContentType(),httpApiConfig.getContent(),context),"");
        }else{
            throw new BizException("不支持的请求体类型!");
        }
    }
    /**
     * @description:  解析返回值
     * @param:
     * @return:
     * @author zhangyy
     * @date: 2024/9/2 16:01
     */
    private Object doLoadResult(Map<String,Object> output) {

        try {
            //判断是否存在这个返回信息的配置
//            if(StringUtils.equals(httpApiConfig.getContentType(), ApiPayloadTypeEnum.APPLICATION_JSON.getCode())){
//               if (Objects.isNull(httpApiConfig.getResultHandle())) {
//                   result=Objects.toString(ApiPayloadResolverManage.getInstance().resolve(httpApiConfig.getContentType(),httpApiConfig.getResult(),output),"");
//               }
//                JSONArray resultHandleJson=JSON.parseObject(httpApiConfig.getResultHandle(),JSONArray.class);
//               if (CollectionUtils.isEmpty(resultHandleJson)) {
//                   result=Objects.toString(ApiPayloadResolverManage.getInstance().resolve(httpApiConfig.getContentType(),httpApiConfig.getResult(),output),"");
//               }
               return Objects.toString(ApiResultResolverManage.getInstance().resolve(httpApiConfig.getContentType(),httpApiConfig.getResult(),output),"");
//            }
        }catch (Exception e){
            //下面这种方式是默认和统一的返回
            log.error("Api 统一返回值解析异常",e);
            return null;
        }

    }


}
