package com.hrsk.cloud.eg.infrastructure.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Map;
import java.util.concurrent.TimeUnit;


/**
 * @author zhangyy
 * @version 1.0
 * @description: Redisutil
 * @date 2024/9/10 10:54
 */
@Component
public class RedisUtil {

    private static final Logger log = LoggerFactory.getLogger(RedisUtil.class);
    @Resource
    private RedisTemplate<String, Object> redisTemplate;

    /**  默认过期时长为24小时，单位：秒 */
    public final static long DEFAULT_EXPIRE = 60 * 60 * 24L;
    /**  过期时长为1小时，单位：秒 */
    public final static long HOUR_ONE_EXPIRE = 60 * 60 * 1L;
    /**  过期时长为6小时，单位：秒 */
    public final static long HOUR_SIX_EXPIRE = 60 * 60 * 6L;
    /**  不设置过期时长 */
    public final static long NOT_EXPIRE = -1L;

    /**
     * 添加值
     * @param key
     * @param value
     * @param expire
     */
    public void set(String key, Object value, long expire) {
        redisTemplate.opsForValue().set(key, value,expire, TimeUnit.MINUTES);
        if (expire != NOT_EXPIRE) {
            expire(key, expire);
        }
    }
    /**
     * 添加值
     * @param key
     * @param value
     * @param
     */
    public void set(String key, Object value) {
        set(key, value, DEFAULT_EXPIRE);
    }

    /**
     * 获取值
     * @param key
     * @param
     * @param
     */
    public Object get(String key, long expire) {
        Object value = redisTemplate.opsForValue().get(key);
        if (expire != NOT_EXPIRE) {
            expire(key, expire);
        }
        return value;
    }
    /**
     * 获取值
     * @param key
     * @param
     * @param
     */
    public Object get(String key) {
        return get(key, NOT_EXPIRE);
    }
    /**
     * 删除key
     * @param key
     * @param
     * @param
     */
    public void delete(String key) {
        redisTemplate.delete(key);
    }
    /**
     * 删除key
     * @param
     * @param
     * @param
     */
    public void delete(Collection<String> keys) {
        redisTemplate.delete(keys);
    }

    /**
     * 获取map中某个key的数据
     * @param
     * @param
     * @param
     */
    public Object hGet(String key, String field) {
        return redisTemplate.opsForHash().get(key, field);
    }
    /**
     * 获取map中的数据
     * @param
     * @param
     * @param
     */
    public Map<String, Object> hGetAll(String key) {
        HashOperations<String, String, Object> hashOperations = redisTemplate.opsForHash();
        return hashOperations.entries(key);
    }

    /**
     * 添加map中的数据
     * @param
     * @param
     * @param
     */
    public void hMSet(String key, Map<String, Object> map) {
        hMSet(key, map, DEFAULT_EXPIRE);
    }

    /**
     * 添加整个map中的数据
     * @param
     * @param
     * @param
     */
    public void hMSet(String key, Map<String, Object> map, long expire) {
        redisTemplate.opsForHash().putAll(key, map);

        if (expire != NOT_EXPIRE) {
            expire(key, expire);
        }
    }
    /**
     * 添加set中的数据
     * @param
     * @param
     * @param
     */
    public void hSet(String key, String field, Object value) {
        hSet(key, field, value, DEFAULT_EXPIRE);
    }

    /**
     * 添加set中的数据
     * @param
     * @param
     * @param
     */
    public void hSet(String key, String field, Object value, long expire) {
        redisTemplate.opsForHash().put(key, field, value);

        if (expire != NOT_EXPIRE) {
            expire(key, expire);
        }
    }

    /**
     * @description: 查看是否过期
     * @param:
     * @return:
     * @author zhangyy
     * @date: 2024/9/10 13:38
     */
    public void expire(String key, long expire) {
        redisTemplate.expire(key, expire, TimeUnit.SECONDS);
    }

    /** 
     * @description: 删除hash 
     * @param:  
     * @return:  
     * @author zhangyy
     * @date: 2024/9/10 13:40
     */ 
    public void hDel(String key, Object... fields) {
        redisTemplate.opsForHash().delete(key, fields);
    }

    /** 
     * @description: 队列推送
     * @param:  
     * @return:  
     * @author zhangyy
     * @date: 2024/9/10 13:40
     */ 
    public void leftPush(String key, Object value) {
        leftPush(key, value, DEFAULT_EXPIRE);
    }

    /** 
     * @description: 队列推送 
     * @param:  
     * @return:  
     * @author zhangyy
     * @date: 2024/9/10 13:40
     */ 
    public void leftPush(String key, Object value, long expire) {
        redisTemplate.opsForList().leftPush(key, value);

        if (expire != NOT_EXPIRE) {
            expire(key, expire);
        }
    }

    /** 
     * @description: 队列弹出
     * @param:  
     * @return:  
     * @author zhangyy
     * @date: 2024/9/10 13:44
     */ 
    public Object rightPop(String key) {
        return redisTemplate.opsForList().rightPop(key);
    }


    /**
     * @description: 是否存在redisKey
     * @param:
     * @return:
     * @author zhangyy
     * @date: 2024/6/30 14:58
     */
    public Boolean getHashKey(String key){
        try {
            if (redisTemplate.hasKey(key)){
                return true;
            }
            return false;
        }catch (Exception e){
            log.error("判断出现错误",e);
            return false;
        }

    }
    /**
     * @description: 删除key
     * @param:
     * @return:
     * @author zhangyy
     * @date: 2024/7/8 21:51
     */
    public Boolean removeKey(String key){
        Boolean delete = redisTemplate.delete(key);
        if (delete){
            return  true;
        }
        return false;
    }


    /**
     * @description: set数据携带了单位
     * @param:
     * @return:
     * @author zhangyy
     * @date: 2024/9/10 15:41
     */
    public void set(String key, String value, int time, TimeUnit timeUnit) {
        try {
            redisTemplate.opsForValue().set(key, value, time, timeUnit);
        }catch (Exception e){
            log.error("redis set errr",e);
        }
    }
}
