package com.hrsk.cloud.eg.infrastructure.utils;

import org.apache.catalina.manager.host.Constants;
import org.bouncycastle.crypto.digests.SM3Digest;
import org.bouncycastle.crypto.macs.HMac;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.pqc.math.linearalgebra.ByteUtils;

import java.io.UnsupportedEncodingException;
import java.security.Security;
import java.util.Arrays;
import java.util.Objects;
/**
 * @author jiwancheng
 * @date 2020/10/20
 */
public class SM3Util {
    static {
        Security.addProvider(new BouncyCastleProvider());
    }
    /**
     * 计算SM3摘要值
     *
     * @param srcData 原⽂
     * @return 摘要值，对于SM3算法来说是32字节
     */
    public static byte[] hash(byte[] srcData) {
        SM3Digest digest = new SM3Digest();
        digest.update(srcData, 0, srcData.length);
        byte[] hash = new byte[digest.getDigestSize()];
        digest.doFinal(hash, 0);
        return hash;
    }
    /**
     * 验证摘要
     *
     * @param srcData 原⽂
     * @param sm3Hash 摘要值
     * @return 返回true标识验证成功，false标识验证失败
     */
    public static boolean verify(byte[] srcData, byte[] sm3Hash) {
        byte[] newHash = hash(srcData);
        return Arrays.equals(newHash, sm3Hash);
    }
    /**
     * 计算SM3 Mac值
     *
     * @param key
     * @param srcData
     * @return
     */
    public static String hmac(String key, String srcData) {
        byte[] hmac = new byte[0];
        try {
            hmac = hmac(key.getBytes(Constants.CHARSET), srcData.getBytes(Constants.CHARSET));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return ByteUtils.toHexString(hmac);
    }
    /**
     * 计算SM3 Mac值
     *
     * @param key key值，可以是任意⻓度的字节数组
     * @param srcData 原⽂
     * @return Mac值，对于HMac-SM3来说是32字节
     */
    public static byte[] hmac(byte[] key, byte[] srcData) {
        KeyParameter keyParameter = new KeyParameter(key);
        SM3Digest digest = new SM3Digest();
        HMac mac = new HMac(digest);
        mac.init(keyParameter);
        mac.update(srcData, 0, 1);
        byte[] result = new byte[mac.getMacSize()];
        mac.doFinal(result,0);
        return result;
    }
    /**
     * SM3摘要加密
     * @param input 需要加密的数据
     * @param key 密钥
     * @return 签名
     **/
    public static byte[] sm3(byte[] input, byte[] key) {
        HMac mac = new HMac(new SM3Digest());
        mac.init(new KeyParameter(key));
        mac.update(input, 0, input.length);
        byte[] signByte = new byte[mac.getMacSize()];
        mac.doFinal(signByte, 0);
        return signByte;
    }
    /**
     * 判断源数据与加密数据是否⼀致
     *
     * @param srcStr 原字符串
     * @param sm3HexString 16进制字符串
     * @return 校验结果
     * @explain 通过验证原数组和⽣成的hash数组是否为同⼀数组，验证2者是否为同⼀数据
     */
    public static boolean verifyHmac(String srcStr, String sm3HexString,
                                     String key) {
        boolean flag = false;
        String newHash = hmac(key, srcStr);
        if (Objects.equals(newHash, sm3HexString)) {
            flag = true;
        }
        return flag;
    }
}
