package com.hrsk.cloud.eg.infrastructure.utils;

import org.apache.catalina.manager.host.Constants;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.StringUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.pqc.math.linearalgebra.ByteUtils;
import org.bouncycastle.util.encoders.Base64;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.Key;
import java.security.SecureRandom;
import java.security.Security;
import java.util.Arrays;
/**
 * @Author Liu Tao
 * @Date 2024 03 18 11 56
 **/
public class SM4Util {
    static {
        Security.addProvider(new BouncyCastleProvider());
    }
    public static final String ALGORITHM_NAME = "SM4";
    private static final String SYSTEM_KEY = "D5AFF760F8A0F095E1268D5EA6A82269";
    /**
     * 加密算法/分组加密模式/分组填充⽅式
     * PKCS5Padding-以8个字节为⼀组进⾏分组加密
     * 定义分组加密模式使⽤：PKCS5Padding
     */
    public static final String ALGORITHM_NAME_ECB_PADDING = "SM4/ECB/PKCS5Padding";
    private static final String ALGORITHM_CBC_PKCS5PADDING = "SM4/CBC/PKCS5Padding";
/**
 * 128-32位16进制；256-64位16进制
 */
public static final int DEFAULT_KEY_SIZE = 128;
    /**
     * ⾃动⽣成密钥
     *
     * @return
     * @explain
     */
    public static String generateKey() {
        try {
            byte[] data = generateKey(DEFAULT_KEY_SIZE);
            return new String(Hex.encodeHex(data, false));
        } catch (Throwable e) {
            return null;
        }
    }
    /**
     * @param keySize
     * @return
     * @throws Exception
     * @explain
     */
    public static byte[] generateKey(int keySize) throws Exception {
        KeyGenerator kg = KeyGenerator.getInstance(ALGORITHM_NAME, BouncyCastleProvider.PROVIDER_NAME);
        kg.init(keySize, new SecureRandom());
        return kg.generateKey().getEncoded();
    }
    /**
     * ⽣成ECB暗号
     *
     * @param algorithmName 算法名称
     * @param mode 模式
     * @param key
     * @return
     * @throws Exception
     * @explain ECB模式（电⼦密码本模式：Electronic codebook）
     */
    private static Cipher generateEcbCipher(String algorithmName, int mode, byte[] key) throws Exception {
        Cipher cipher = Cipher.getInstance(algorithmName, BouncyCastleProvider.PROVIDER_NAME);
        Key sm4Key = new SecretKeySpec(key, ALGORITHM_NAME);
        cipher.init(mode, sm4Key);
        return cipher;
    }
    /**
     * sm4加密
     *
     * @param hexKey 16进制密钥（忽略⼤⼩写）
     * @param paramStr 待加密字符串
     * @return 返回16进制的加密字符串
     * @explain 加密模式：ECB
     * 密⽂⻓度不固定，会随着被加密字符串⻓度的变化⽽变化
     */
    public static String encryptEcb(String hexKey, String paramStr) {
        try {
            // 16进制字符串-->byte[]
            byte[] keyData = ByteUtils.fromHexString(hexKey);
            // String-->byte[]
            byte[] srcData = paramStr.getBytes(Constants.CHARSET);
            // 加密后的数组
            byte[] cipherArray = encrypt_Ecb_Padding(keyData, srcData);
            // byte[]-->hexString
            return ByteUtils.toHexString(cipherArray);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    /**
     * 将明⽂字符串加密为BASE64
     *
     * @param data 要加密的明⽂
     * @param secretKey 16位密钥
     * @return 加密后BASE64密⽂
     * @throws Exception 加密异常
     */
    public static String encryptStrToBase64(String data, String secretKey
    ) throws Exception {
        return Base64.toBase64String(encrypt(data.getBytes(Constants.CHARSET), secretKey));
    }
/**
 * 加密
 *
 * @param data 要加密的明⽂
 * @param secretKey 16位密钥
 * @return 加密后的密⽂
 * @throws Exception 加密异常
 */
public static byte[] encrypt(byte[] data, String secretKey) throws Exception {
        return sm4(data, secretKey.getBytes(Constants.CHARSET), Cipher.ENCRYPT_MODE);
    }
    /**
     * 将BASE64密⽂解密为明⽂字符串
     *
     * @param base64Encrypt 要解密的BASE64密⽂
     * @param secretKey 16位密钥
     * @return 解密后的明⽂字符串
     * @throws Exception 加密异常
     */
    public static String decryptBase64ToStr(String base64Encrypt, String
            secretKey) throws Exception {
        byte[] decryptByte = decrypt(Base64.decode(base64Encrypt), secretKey);
        return new String(decryptByte, StandardCharsets.UTF_8);
    }
    /**
     * 解密
     *
     * @param data 要解密的密⽂
     * @param secretKey 16位密钥
     * @return 解密后的明⽂
     * @throws Exception 解密异常
     */
    public static byte[] decrypt(byte[] data, String secretKey) throws Exception {
        return sm4(data, secretKey.getBytes(Constants.CHARSET), Cipher.DECRYPT_MODE);
    }
    /**
     * SM4对称加解密
     *
     * @param input 需要加密或解密的数据
     * @param key 密钥
     * @param mode Cipher.ENCRYPT_MODE - 加密；Cipher.DECRYPT_MODE - 解密
     * @return 密⽂（加密模式）或明⽂（解密模式）
     * @throws Exception 加解密异常
     */
    private static byte[] sm4(byte[] input, byte[] key, int mode) throws
            Exception{
        SecretKeySpec sm4Key = new SecretKeySpec(key, ALGORITHM_NAME);
        Cipher cipher = Cipher.getInstance(ALGORITHM_CBC_PKCS5PADDING, BouncyCastleProvider.PROVIDER_NAME);
        cipher.init(mode, sm4Key, new IvParameterSpec(new byte[16]));
        return cipher.doFinal(input);
    }
    /**
     * 加密模式之Ecb
     *
     * @param key
     * @param data
     * @return
     * @throws Exception
     * @explain
     */
    public static byte[] encrypt_Ecb_Padding(byte[] key, byte[] data) throws Exception {
        Cipher cipher = generateEcbCipher(ALGORITHM_NAME_ECB_PADDING, Cipher.ENCRYPT_MODE, key);
        return cipher.doFinal(data);
    }
    /**
     * sm4解密
     *
     * @param hexKey 16进制密钥
     * @param cipherText 16进制的加密字符串（忽略⼤⼩写）
     * @return 解密后的字符串
     * @throws Exception
     * @explain 解密模式：采⽤ECB
     */
    public static String decryptEcb(String hexKey, String cipherText) {
        // ⽤于接收解密后的字符串
        String decryptStr = "";
        // hexString-->byte[]
        byte[] keyData = ByteUtils.fromHexString(hexKey);
        // hexString-->byte[]
        byte[] cipherData = ByteUtils.fromHexString(cipherText);
        // 解密
        try {
            byte[] srcData = decrypt_Ecb_Padding(keyData, cipherData);
            // byte[]-->String
            decryptStr = new String(srcData, Constants.CHARSET);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return decryptStr;
    }
    /**
     * 解密
     *
     * @param key
     * @param cipherText
     * @return
     * @throws Exception
     * @explain
     */
    public static byte[] decrypt_Ecb_Padding(byte[] key, byte[] cipherText) throws Exception {
        Cipher cipher = generateEcbCipher(ALGORITHM_NAME_ECB_PADDING, Cipher.DECRYPT_MODE, key);
        return cipher.doFinal(cipherText);
    }
    /**
     * 校验加密前后的字符串是否为同⼀数据
     *
     * @param hexKey 16进制密钥（忽略⼤⼩写）
     * @param cipherText 16进制加密后的字符串
     * @param paramStr 加密前的字符串
     * @return 是否为同⼀数据
     * @throws Exception
     * @explain
     */
    public static boolean verifyEcb(String hexKey, String cipherText, String paramStr) throws Exception {
        // ⽤于接收校验结果
        boolean flag = false;
        // hexString-->byte[]
        byte[] keyData = ByteUtils.fromHexString(hexKey);
        // 将16进制字符串转换成数组
        byte[] cipherData = ByteUtils.fromHexString(cipherText);
        // 解密
        byte[] decryptData = decrypt_Ecb_Padding(keyData, cipherData);
        // 将原字符串转换成byte[]
        byte[] srcData = paramStr.getBytes(Constants.CHARSET);
        // 判断2个数组是否⼀致
        flag = Arrays.equals(decryptData, srcData);
        return flag;
    }
/**
 * ⽤系统密钥加密
 *
 * @param str
 * @return
 */
public static String encryptSystemKey(String str) {
    if (StringUtils.isEmpty(str)) {
        return null;
    }
    return SM4Util.encryptEcb(SYSTEM_KEY, str);
}
    /**
     * ⽤系统密钥解密
     *
     * @param str
     * @return
     */
    public static String decryptSystemKey(String str) {
        if (StringUtils.isEmpty(str)) {
            return null;
        }
        return SM4Util.decryptEcb(SYSTEM_KEY, str);
    }
 }
